#ifndef Motion_Oscillator_tpp
#define Motion_Oscillator_tpp

#include <ArduinoLog.h>

namespace LinearPositionControl { namespace Motion {

// Oscillator

template <class PositionTracker>
Oscillator<PositionTracker>::Oscillator(Components::Motor &motor, PositionTracker &positionTracker) :
  motor(motor), positionTracker(positionTracker) {
}

template <class PositionTracker>
void Oscillator<PositionTracker>::setup() {
  if (setupCompleted) return;

  motor.setup();
  positionTracker.setup();

  state.setup(State::initializing);

  setupCompleted = true;
}

template <class PositionTracker>
void Oscillator<PositionTracker>::update() {
  positionTracker.update();
  if (positionTracker.state.current() != PositionTracker::State::tracking) return;

  switch (state.current()) {
    case State::operating:
      updateOperating();
      break;
    case State::initializing:
      updateInitializing();
      break;
  }
}

template <class PositionTracker>
void Oscillator<PositionTracker>::updateInitializing() {
  Log.notice(F("Oscillating the motor between the limits!" CR));
  motor.opposite(); // we assume that the actuator is either floating or at the left limit, which is true after direction calibration
  state.update(State::operating);
}

template <class PositionTracker>
void Oscillator<PositionTracker>::updateOperating() {
  if (positionTracker.atLeftLimit()) {
      if (positionTracker.position.justChanged()) Log.trace(F("Reached left limit. Moving forwards!" CR));
      motor.forwards();
  } else if (positionTracker.atRightLimit()) {
      if (positionTracker.position.justChanged()) Log.trace(F("Reached right limit. Moving backwards!" CR));
      motor.backwards();
  }
}

} }

#endif

